--[[
    Botman - A collection of scripts for managing 7 Days to Die servers
    Copyright (C) 2020  Matthew Dwyer
	           This copyright applies to the Lua source code in this Mudlet profile.
    Email     smegzor@gmail.com
    URL       http://botman.nz
    Source    https://bitbucket.org/mhdwyer/botman
--]]

-- enable debug to see where the code is stopping. Any error will be after the last debug line.
local debug = false -- should be false unless testing


function CheckInventory()
	local temp, table1, table2, items, d1, changes, tmp, max, search, k, v, i, a, b

	-- do a quick sanity check to prevent a rare fault causing this to get stuck
	for k, v in pairs(igplayers) do
		if players[k] == nil then
			igplayers[k] = nil
		end
	end

	for k, v in pairs(igplayers) do
		if debug then dbug(k .. " " .. v.name) end

		tmp = {}
		tmp.playerAccessLevel = accessLevel(k)
		tmp.ban = false
		tmp.banReason = ""
		tmp.timeout = false
		tmp.timeoutReason = ""
		tmp.move = false
		tmp.moveTo = ""
		tmp.moveReason = ""
		tmp.newPlayer = false
		tmp.badItemFound = false
		tmp.itemsFound = ""
		tmp.badItem = ""
		tmp.badItemAction = ""
		tmp.watchPlayer = players[k].watchPlayer
		tmp.newItems = ""
		tmp.delta = 0
		tmp.inventoryChanged = false
		tmp.flags = ""
		tmp.dbFlag = ""
		tmp.stopProcessing = false

		if v.inLocation ~= "" then
			if locations[v.inLocation].watchPlayers and tmp.playerAccessLevel > 2 then
				tmp.watchPlayer = true
			end
		end

		temp = {}
		items = {}
		changes = {}
		players[k].overstack = false
		players[k].overstackItems = ""
		v.illegalInventory = false

		if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

		if (tonumber(players[k].timeOnServer) + tonumber(v.sessionPlaytime) < (tonumber(server.newPlayerTimer) * 60) ) then
			tmp.newPlayer = true
		else
			tmp.newPlayer = false
		end

		if tmp.newPlayer == true then
			tmp.flags = "|NEW|"
		else
			if (tmp.watchPlayer == true) then
				tmp.flags = "|WAT|" -- watched
			end
		end

		if v.raiding == true then
			tmp.flags = tmp.flags .. " at base of " .. v.raidingBase .. " " .. players[v.raidingBase].name .. " "
			tmp.dbFlag = "R"
		end

		if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

		if (v.inventory ~= "") then
			if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end
			table1 = string.split(v.inventory, "|")

			max = table.maxn(table1)
			for i = 1, max do
				if table1[i] ~= "" then
					table2 = string.split(table1[i], ",")
					tmp.badItemFound = false
					tmp.badItem = ""

					-- check for wildcard items in badItems and search for those
					for a,b in pairs(badItems) do
						if string.find(a, "*", nil, true) then
							search = a:gsub('%W','')

							if string.find(table2[2], search) then
								tmp.badItemFound = true
								tmp.badItem = table2[2]
								tmp.badItemAction = b.action
								break
							end
						end
					end

					if (badItems[table2[2]] or tmp.badItemFound) and (tmp.playerAccessLevel > 2 or botman.ignoreAdmins == false) and (not players[k].ignorePlayer) and (server.gameType ~= "cre") then
						tmp.dbFlag = tmp.dbFlag .. "B"
						v.illegalInventory = true

						if badItems[table2[2]] then
							tmp.badItemFound = true
							tmp.badItemAction = badItems[table2[2]].action
							tmp.badItem = table2[2]

							if tmp.itemsFound == "" then
								tmp.itemsFound = table2[2] .. "(" .. table2[1] .. ")"
							else
								tmp.itemsFound = tmp.itemsFound .. ", " .. table2[2] .. "(" .. table2[1] .. ")"
							end
						else
							if tmp.badItemFound then
								if tmp.itemsFound == "" then
									tmp.itemsFound = tmp.badItem
								else
									tmp.itemsFound = tmp.itemsFound .. ", " .. tmp.badItem
								end
							end
						end

						if tmp.badItemAction == "ban" then
							tmp.ban = true
							tmp.banReason = "Bad item found in inventory"

							if v.raiding then
								tmp.banReason = "Bad item found in inventory while base raiding"
							end
						end

						if tmp.badItemAction == "exile" then
							tmp.move = true
							tmp.moveTo = "exile"

							if tmp.moveReason == nil then
								tmp.moveReason = "Bad items found " .. tmp.badItem .. "(" .. table2[1] .. ")"

								if v.raiding then
									tmp.moveReason = "Bad items found while raiding "
								end
							else
								tmp.moveReason = tmp.moveReason .. ", " .. tmp.badItem .. "(" .. table2[1] .. ")"
							end
						end

						if tmp.badItemAction == "timeout" then
							tmp.timeout = true
							tmp.timeoutReason = "Bad item found in inventory"
						end
					end

					if items[table2[2]] == nil then
						items[table2[2]] = {}
						items[table2[2]].item = table2[2]
						items[table2[2]].quantity = tonumber(table2[1])
						items[table2[2]].quality = tonumber(table2[3])
						items[table2[2]].dupe = 0

						if tonumber(table2[1]) == 1 then
							items[table2[2]].dupe = 1
						end
					else
						items[table2[2]].quantity = items[table2[2]].quantity + tonumber(table2[1])

						if tonumber(table2[1]) == 1 then
							items[table2[2]].dupe = items[table2[2]].dupe + 1
						end
					end

					-- stack monitoring
					if (stackLimits[table2[2]] ~= nil) and (tmp.playerAccessLevel > 2 or botman.ignoreAdmins == false) and (server.gameType ~= "cre") and (not players[k].ignorePlayer) and not server.allowOverstacking then
						if tonumber(table2[1]) > tonumber(stackLimits[table2[2]].limit) * 2 and tonumber(table2[1]) > 1000 then
							if (players[k].overstackScore < 0) then
								players[k].overstackScore = 0
							end

							if not server.allowOverstacking then
								players[k].overstack = true
								players[k].overstackItems = players[k].overstackItems .. " " .. table2[2] .. " (" .. table2[1] .. ")"
								players[k].overstackScore = players[k].overstackScore + 1
							end
						end

						-- instant ban for a full stack of any of these if a new player
						if tonumber(table2[1]) >= tonumber(stackLimits[table2[2]].limit) and tmp.newPlayer == true then
							if (table2[2] == "tnt" or table2[2] == "mineAirFilter" or table2[2] == "mineHubcap" or table2[2] == "rScrapIronPlateMine") then
								tmp.ban = true
								tmp.banReason = "Banned for excessive amounts of " .. table2[2] .. "(" .. table2[1] .. ")."
							end
						end
					end
				end
			end

			if v.raiding and tmp.timeout then
				players[k].exiled = true
				if tmp.playerAccessLevel > 2 then players[k].silentBob = true end
				players[k].canTeleport = false

				irc_chat(server.ircMain, "Exiling " .. v.name .. " detected with bad inventory while raiding.")
				irc_chat(server.ircAlerts, server.gameDate .. " exiling " .. v.name .. " detected with bad inventory while raiding.")
				irc_chat(server.ircAlerts, server.gameDate .. " Items detected: " .. tmp.itemsFound)
			end

			if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

			for a, b in pairs(items) do
				if tmp.newPlayer and b.dupe > 15 then
					if not v.dupeItem then
						v.dupeItem = b.item
						irc_chat(server.ircWatch, "New player " .. players[k].name .. " has " .. b.dupe .. " x 1 of " .. b.item)
					end

					if b.item ~= v.dupeItem then
						v.dupeItem = b.item
						irc_chat(server.ircWatch, "New player " .. players[k].name .. " has " .. b.dupe .. " x 1 of " .. b.item)
					end
				end

				if (tmp.newPlayer == true and v.skipExcessInventory ~= true) and (server.gameType ~= "cre") then
					cursor,errorString = conn:execute("SELECT * FROM memRestrictedItems where item = '" .. escape(b.item) .. "' and accessLevel < " .. tmp.playerAccessLevel)
					rows = cursor:numrows()

					if tonumber(rows) > 0 then
						row = cursor:fetch({}, "a")

						if tonumber(b.quantity) > tonumber(row.qty) and (not players[k].ignorePlayer) then
							if row.action == "timeout" and server.gameType ~= "pvp" then
								tmp.timeout = true

								if tmp.timeoutReason == nil then
									tmp.timeoutReason = "excessive inventory for a new player " .. b.item .. "(" .. b.quantity .. ")"
								else
									tmp.timeoutReason = tmp.timeoutReason .. ", " .. b.item .. "(" .. b.quantity .. ")"
								end
							end

							if row.action == "exile" then
								tmp.move = true
								tmp.moveTo = "exile"

								if tmp.moveReason == nil then
									tmp.moveReason = "Restricted items found " .. b.item .. "(" .. b.quantity .. ")"

									if v.raiding then
										tmp.moveReason = "Restricted items found while raiding "
									end
								else
									tmp.moveReason = tmp.moveReason .. ", " .. b.item .. "(" .. b.quantity .. ")"
								end
							end

							if row.action == "ban" then
								tmp.ban = true

								if tmp.banReason == nil then
									tmp.banReason = "bad inventory " .. b.item .. "(" .. b.quantity .. ")"
								else
									tmp.banReason = tmp.banReason .. ", " .. b.item .. "(" .. b.quantity .. ")"
								end
							end

							if locations[row.action] then
								tmp.move = true
								tmp.moveTo = row.action

								if moveReason == nil then
									tmp.moveReason = "excessive inventory for a new player " .. b.item .. "(" .. b.quantity .. ")"
								else
									tmp.moveReason = tmp.moveReason .. ", " .. b.item .. "(" .. b.quantity .. ")"
								end
							end

							if row.action == "watch" then
								if v.inLocation ~= "" then
									irc_chat(server.ircWatch, "Player " .. v.name .. " in " .. v.inLocation .. " " .. " has " .. b.quantity .. " of " .. b.item .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos)
								else
									irc_chat(server.ircWatch, "Player " .. v.name .. " " .. " has " .. b.quantity .. " of " .. b.item .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos)
								end
							end
						end
					end
				end
			end

			if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

			if tablelength(invTemp[k]) == 0 then
				invTemp[k] = items
			end

			if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

			tmp.flag = ""

			for a, b in pairs(invTemp[k]) do
				if items[b.item] == nil then
					items[b.item] = {}
					items[b.item].item = b.item
					items[b.item].quantity = 0

					if badItems[b.item] then
						tmp.flag = "B"
					end

					if v.raiding then
						tmp.flag = tmp.flag .. "R"
					end
				end

				if tonumber(b.quantity) ~= tonumber(items[a].quantity) then
					tmp.inventoryChanged = true
					table.insert(changes, { b.item, tonumber(items[a].quantity) - tonumber(b.quantity) } )
					conn:execute("INSERT INTO inventoryChanges (steam, item, delta, x, y, z, session, flag) VALUES (" .. k .. ",'" .. escape(b.item) .. "'," .. tonumber(items[a].quantity) - tonumber(b.quantity) .. "," .. v.xPos .. "," .. v.yPos .. "," .. v.zPos .. "," .. players[k].sessionCount .. ",'" .. tmp.flag .. "')")

					if server.logInventory then
						logInventoryChanges(k, b.item, tonumber(items[a].quantity) - tonumber(b.quantity), v.xPos, v.yPos, v.zPos,players[k].sessionCount, tmp.flag)
					end

					v.afk = os.time() + 900

					if (items[a] == nil) then
						d1 = 0
					else
						d1 = tonumber(items[a].quantity)
					end

					tmp.delta = d1 - tonumber(b.quantity)
					if tonumber(tmp.delta) > 0 then
						tmp.delta = "+" .. tmp.delta
					else
						tmp.delta = tmp.delta

						-- list beds for this player if they drop 1 bed
						if b.item == "bedroll" and tmp.delta == -1 and server.coppi then
							listPlayerBed(k)
						end
					end

					if (players[k].watchPlayer or tmp.watchPlayer) then
						cursor,errorString = conn:execute("SELECT * FROM memRestrictedItems where item = '" .. escape(b.item) .. "' and action = 'watch'")
						row = cursor:fetch({}, "a")
						if row then
							if (b.item == row.item) and not string.find(newItems, b.item, nil, true) then
								tmp.newItems = tmp.newItems .. row.item .. " (" .. tmp.delta .. "), "
							end
						end

						if tonumber(tmp.delta) > 0 and tmp.newPlayer == true and not string.find(tmp.newItems, b.item, nil, true) then
							tmp.newItems = tmp.newItems .. b.item .. " (" .. tmp.delta .. "), "
						end

						if (b.item == "keystoneBlock") and not string.find(tmp.newItems, b.item, nil, true) then
							tmp.newItems = tmp.newItems .. "keystoneBlock (" .. tmp.delta .. "), "
						end
					end
				end
			end

			if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

			if (players[k].watchPlayer) and not server.disableWatchAlerts then
				if tmp.newItems ~= "" then
					alertAdmins("Watched player " .. v.id .. " " .. v.name .. " " .. tmp.newItems)
				end
			end

			if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

			if tmp.inventoryChanged == true or (v.oldBelt ~= v.belt) or (v.oldPack ~= v.pack) or (v.oldEquipment ~= v.equipment) then
				conn:execute("INSERT INTO inventoryTracker (steam, x, y, z, session, belt, pack, equipment) VALUES (" .. k .. "," .. v.xPos .. "," .. v.yPos .. "," .. v.zPos .. "," .. players[k].sessionCount .. ",'" .. escape(v.belt) .. "','" .. escape(v.pack) .. "','" .. escape(v.equipment) .. "')")
				invTemp[k] = items

				if tmp.inventoryChanged == true then
					if players[k].timeOnServer == nil or v.watchPlayer or v.raiding or tmp.watchPlayer then
						for q, w in pairs(changes) do
							if v.inLocation ~= "" then
								irc_chat(server.ircWatch, string.trim(botman.serverTime .. " " .. server.gameDate .. " " .. v.name .. " " .. tmp.flags .. " in " .. v.inLocation .. " " .. w[1] .. "  " .. w[2] .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos))
							else
								irc_chat(server.ircWatch, string.trim(botman.serverTime .. " " .. server.gameDate .. " " .. v.name .. " " .. tmp.flags .. " " .. w[1] .. "  " .. w[2] .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos))
							end
						end
					else
						if tmp.playerAccessLevel > 2 and tonumber(players[k].timeOnServer) < tonumber(server.newPlayerTimer)  then
							for q, w in pairs(changes) do
								if v.inLocation ~= "" then
									irc_chat(server.ircWatch, string.trim(botman.serverTime .. " " .. server.gameDate .. " " .. v.name .. " " .. tmp.flags .. " in " .. v.inLocation .. " " .. w[1] .. "   " .. w[2] .. "  @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos))
								else
									irc_chat(server.ircWatch, string.trim(botman.serverTime .. " " .. server.gameDate .. " " .. v.name .. " " .. tmp.flags .. " " .. w[1] .. "   " .. w[2] .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos))
								end
							end
						end
					end
				end
			end
		end

		v.oldBelt = v.belt

		if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

		tmp.changes = nil

		if (players[k].overstack == false) then
			players[k].overstackScore = 0
		end

		if not server.allowOverstacking and (server.gameType ~= "cre") then
			if (players[k].overstack == true) and (tmp.playerAccessLevel > 2 or botman.ignoreAdmins == false) then
				message("pm " .. k .. " [" .. server.chatColour .. "]You are overstacking items in your inventory - " .. players[k].overstackItems .. "[-]")
				if v.inLocation ~= "" then
					irc_chat(server.ircWatch, botman.serverTime .. " " .. server.gameDate .. " " .. v.name .. " is overstacking " .. players[k].overstackItems .. " in " .. v.inLocation .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos)
				else
					irc_chat(server.ircWatch, botman.serverTime .. " " .. server.gameDate .. " " .. v.name .. " is overstacking " .. players[k].overstackItems .. "  at " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos)
				end
			end

			if (tonumber(players[k].overstackScore) == 2) and (players[k].botTimeout == false) then
				message("pm " .. k .. " [" .. server.chatColour .. "]If you do not stop overstacking, you will be sent to timeout.  Fix your inventory now.[-]")
			end

			if tonumber(players[k].overstackScore) > 4 and (players[k].botTimeout == false) then
				players[k].botTimeout = true
				players[k].xPosTimeout = players[k].xPos
				players[k].yPosTimeout = players[k].yPos
				players[k].zPosTimeout = players[k].zPos

				message("say [" .. server.chatColour .. "]" .. v.name .. " is in timeout for ignoring overstack warnings.[-]")
				message("pm " .. k .. " [" .. server.chatColour .. "]You are still overstacking items. You will stay in timeout until you are not overstacking.[-]")
				irc_chat(server.ircWatch, botman.serverTime .. " " .. server.gameDate .. " [TIMEOUT] " .. k .. " " .. v.name .. " is in timeout for overstacking the following " .. players[k].overstackItems)
				irc_chat(server.ircAlerts, botman.serverTime .. " " .. server.gameDate .. " [TIMEOUT] " .. k .. " " .. v.name .. " is in timeout for overstacking the following " .. players[k].overstackItems)

				conn:execute("INSERT INTO events (x, y, z, serverTime, type, event, steam) VALUES (" .. v.xPos .. "," .. v.yPos .. "," .. v.zPos .. ",'" .. botman.serverTime .. "','timeout','Player " .. escape(v.name) .. " is in timeout for overstacking the following " .. escape(players[k].overstackItems) .. "'," .. k .. ")")
			end
		end

		if (ban == true) and (server.gameType ~= "cre") then
			if tmp.playerAccessLevel > 2 then
				tmp.stopProcessing = true
				banPlayer(k, "1 year", tmp.banReason, "")

				message("say [" .. server.chatColour .. "]Banning player " .. v.name .. " 1 year for suspected inventory cheating.[-]")
				irc_chat(server.ircMain, "[BANNED] Player " .. k .. " " .. v.name .. " has has been banned for " .. tmp.banReason .. ".")
				irc_chat(server.ircAlerts, botman.serverTime .. " " .. server.gameDate .. " [BANNED] Player " .. k .. " " .. v.name .. " has has been banned for 1 year for " .. tmp.banReason .. ".")
				conn:execute("INSERT INTO events (x, y, z, serverTime, type, event, steam) VALUES (" .. v.xPos .. "," .. v.yPos .. "," .. v.zPos .. ",'" .. botman.serverTime .. "','ban','Player " .. k .. " " .. escape(v.name) .. " has has been banned for 1 year for " .. escape(tmp.banReason) .. ".'," .. k .. ")")

				if botman.db2Connected then
					-- copy in bots db
					connBots:execute("INSERT INTO events (server, serverTime, type, event, steam) VALUES ('" .. escape(server.serverName) .. "','" .. botman.serverTime .. "','ban','Player " .. k .. " " .. escape(v.name) .. " has has been banned for 1 year for " .. escape(tmp.banReason) .. ".'," .. k .. ")")
				end

				irc_chat(server.ircAlerts, server.gameDate .. " Items detected: " .. tmp.itemsFound)
			end
		end

		--if (tmp.timeout == true) and (server.gameType ~= "cre") and not tmp.stopProcessing then
			--tmp.stopProcessing = true
			--v.illegalInventory = true
			--conn:execute("INSERT INTO inventoryTracker (steam, x, y, z, session, belt, pack, equipment) VALUES (" .. k .. "," .. v.xPos .. "," .. v.yPos .. "," .. v.zPos .. "," .. players[k].sessionCount .. ",'" .. escape(v.belt) .. "','" .. escape(v.pack) .. "','" .. escape(v.equipment) .. "')")
			--timeoutPlayer(k, tmp.timeoutReason, true)
		--end

		if tmp.move == true and (server.gameType ~= "cre") and not tmp.stopProcessing then
			if not players[k].exiled then
				players[k].exiled = true
				if tmp.playerAccessLevel > 2 then players[k].silentBob = true end
				players[k].canTeleport = false
				irc_chat(server.ircMain, "Moving player " .. k .. " " .. v.name .. " to " .. tmp.moveTo .. " for " .. tmp.moveReason .. ".")
				irc_chat(server.ircAlerts, server.gameDate .. " moving player " .. k .. " " .. v.name .. " to " .. tmp.moveTo .. " for " .. tmp.moveReason .. ".")
				conn:execute("INSERT INTO events (x, y, z, serverTime, type, event, steam) VALUES (" .. v.xPos .. "," .. v.yPos .. "," .. v.zPos .. ",'" .. botman.serverTime .. "','exile','Player " .. k .. " " .. escape(v.name) .. " has has been exiled to " .. escape(tmp.moveTo) .. " for " .. escape(tmp.moveReason) .. ".'," .. k .. ")")
				irc_chat(server.ircAlerts, server.gameDate .. " Items detected: " .. tmp.itemsFound)
				teleport("tele " .. k .. " " .. locations[tmp.moveTo].x .. " " .. locations[tmp.moveTo].y + 1 .. " " .. locations[tmp.moveTo].z, k)
				message("say [" .. server.chatColour .. "]Sending player " .. v.name .. " to " .. tmp.moveTo .. " for " .. tmp.moveReason .. ".[-]")
			end

			tmp.stopProcessing = true
		end

		if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

		if (not players[k].ignorePlayer) and (server.gameType ~= "cre") and not players[k].botTimeout then
			if tmp.itemsFound ~= "" then
				v.illegalInventory = true

				if (players[k].timeout == false) and (tmp.playerAccessLevel > 2 or botman.ignoreAdmins == false) and not tmp.stopProcessing then
					--players[k].timeout = true
					players[k].botTimeout = true
					players[k].xPosTimeout = players[k].xPos
					players[k].yPosTimeout = players[k].yPos
					players[k].zPosTimeout = players[k].zPos

					if tmp.playerAccessLevel > 2 then players[k].silentBob = true end
					message("say [" .. server.chatColour .. "]" .. v.name .. " is in timeout for uncraftable items " .. tmp.itemsFound .. ".[-]")
					message("pm " .. k .. " [" .. server.chatColour .. "]You have items in your inventory that are not permitted.[-]")
					message("pm " .. k .. " [" .. server.chatColour .. "]You must drop them if you wish to return to the game.[-]")

					irc_chat(server.ircMain, v.name .. " detected with uncraftable " .. tmp.itemsFound)
					irc_chat(server.ircAlerts, botman.serverTime .. " " .. server.gameDate .. " " .. v.name .. " detected with uncraftable " .. tmp.itemsFound)
					conn:execute("INSERT INTO events (x, y, z, serverTime, type, event) VALUES (" .. igplayers[k].xPos .. "," .. igplayers[k].yPos .. "," .. igplayers[k].zPos .. ",'" .. botman.serverTime .. "','timeout','Player " .. escape(v.name) .. " detected with uncraftable inventory " .. escape(tmp.itemsFound) .. "')")

					if botman.db2Connected then
						-- copy in bots db
						connBots:execute("INSERT INTO events (server, serverTime, type, event, steam) VALUES ('" .. escape(server.serverName) .. "','" .. botman.serverTime .. "','timeout','Player " .. escape(v.name) .. " detected with uncraftable inventory " .. escape(tmp.itemsFound) .. "')")
					end
				end
			end
		end

		if  debug then dbug("debug check inventory line " .. debugger.getinfo(1).currentline, true) end

		if players[k].botTimeout == true and v.illegalInventory == false and players[k].overstack == false then
			players[k].silentBob = false
			players[k].overstackScore = 0
			message("pm " .. k .. " [" .. server.chatColour .. "]You are free to play again.[-]")
			players[k].xPosOld = 0
			players[k].yPosOld = 0
			players[k].zPosOld = 0
			v.lastLocation = ""
			gmsg(server.commandPrefix .. "return " .. k)
		end
	end

	if debug then dbug("check inventory end") end
end


function readInventorySlot()
	local slot, item, quantity, quality -- timestamp, , words, dupeTest, pos

	if server.useAllocsWebAPI then
		return
	end

	if not (string.find(line, "Slot") and string.find(line, ": ")) then
		-- abort if the line isn't actually a player's inventory
		return
	end

	--timestamp = os.time()
	item = ""
	slot = ""
	quantity = 0
	quality = 0
	--words = {}
	--dupeTest = {}

	--for word in line:gmatch("%w+") do table.insert(words, word) end

	slot = string.sub(line, string.find(line, "Slot") + 5, string.find(line, ": ") - 1)

	if string.find(line, "*") then
		slot = tonumber(slot)
		quantity = tonumber(string.sub(line, string.find(line, ":") + 2, string.find(line, "*") - 2))
		item = string.trim(string.sub(line, string.find(line, "* ") + 2))
	else
		quantity = 1
	end

	if string.find(line, "quality:") then
		quality = string.trim(string.sub(line, string.find(line, "quality: ") + 9))

		if string.find(line, "* ") then
			item = string.trim(string.sub(line, string.find(line, "* ") + 2, string.find(line, "quality:") - 4))
		else
			item = string.trim(string.sub(line, string.find(line, ": ") + 2, string.find(line, "quality:") - 4))
		end
	else
		if item == "" then
			item = string.trim(string.sub(line, string.find(line, ": ") + 2))
		end
	end

	if (invScan == "belt") then
		igplayers[invCheckID].inventory = igplayers[invCheckID].inventory .. quantity .. "," .. item .. "," .. quality .. "|"
		igplayers[invCheckID].belt = igplayers[invCheckID].belt .. slot .. "," .. quantity .. "," .. item .. "," .. quality .. "|"
	end

	if (invScan == "bagpack") then
		igplayers[invCheckID].inventory = igplayers[invCheckID].inventory .. quantity .. "," .. item .. "," .. quality .. "|"
		igplayers[invCheckID].pack = igplayers[invCheckID].pack .. slot .. "," .. quantity .. "," .. item .. "," .. quality .. "|"
	end

	if (invScan == "equipment") then
		igplayers[invCheckID].equipment = igplayers[invCheckID].equipment .. slot .. "," .. item .. "," .. quality .. "|"
	end

	if not server.useAllocsWebAPI then
		deleteLine()
	end
end
